﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Controls.Menus;
using System.Collections;
using System.Windows.Data;
using System.Collections.Generic;

namespace XamRibbonCombo
{
    public class ComboBoxTool : RibbonTool
    {
        #region Overrides

        protected override RibbonToolBaseControl ResolveToolControl()
        {
            return new ComboBoxToolControl(this);
        }

        public override RibbonToolSizingMode MaximumSize
        {
            get
            {
                return RibbonToolSizingMode.ImageOnly;
            }
            set
            {
                base.MaximumSize = value;
            }
        }

        #endregion // Overrides

        #region DisplayMemberPath Property

        public String DisplayMemberPath { get; set; }

        #endregion // DisplayMemberPath Property

        #region ItemsSource Property

        public IEnumerable ItemsSource
        {
            get { return (IEnumerable)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }

        public static readonly DependencyProperty ItemsSourceProperty =
            DependencyProperty.Register("ItemsSource", typeof(IEnumerable),
            typeof(ComboBoxTool), null);

        #endregion // ItemsSource Property

        #region SelectedItem Property

        public object SelectedItem
        {
            get { return GetValue(SelectedItemProperty); }
            set { SetValue(SelectedItemProperty, value); }
        }

        public static readonly DependencyProperty SelectedItemProperty =
            DependencyProperty.Register("SelectedItem", typeof(object),
            typeof(ComboBoxTool), new PropertyMetadata(new PropertyChangedCallback(OnSelectionChanged)));

        private static void OnSelectionChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ComboBoxTool tool = (ComboBoxTool)d;

            if (tool.SelectionChanged != null)
            {
                List<object> addedItems = new List<object>();
                List<object> removedItems = new List<object>();
                if (e.OldValue != null)
                {
                    removedItems.Add(e.OldValue);
                }
                if (e.NewValue != null)
                {
                    addedItems.Add(e.NewValue);
                }

                tool.SelectionChanged(tool, new SelectionChangedEventArgs(removedItems, addedItems));
            }
        }        

        #endregion // SelectedItem property

        #region Selection Changed event

        public event SelectionChangedEventHandler SelectionChanged;

        #endregion // Selection Changed event
    }

    public class ComboBoxToolControl : RibbonToolBaseControl, IRibbonControl
    {
        private ComboBox _combo;

        public ComboBoxToolControl()
        {
            this.DefaultStyleKey = typeof(ComboBoxToolControl);            
        }

        public ComboBoxToolControl(RibbonToolBase tool)
            : base(tool)
        {
            this.DefaultStyleKey = typeof(ComboBoxToolControl);
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            this._combo = GetTemplateChild("Combo") as ComboBox;
            if (this._combo != null)
            {
                this._combo.ItemsSource = ((ComboBoxTool)this.Tool).ItemsSource;
                
                Binding b = new Binding("SelectedItem");
                b.Mode = BindingMode.TwoWay;
                b.Source = this.Tool;
                this._combo.SetBinding(ComboBox.SelectedItemProperty, b);
                this._combo.DisplayMemberPath = ((ComboBoxTool)this.Tool).DisplayMemberPath;
            }
        }
    }
}
